/* $Id: search.c,v 1.2 2002/06/28 23:16:13 thrull Exp $ */

/*
 * (C) Copyright 2001-2002 Igor Popik. Released under terms GPL license.
 */

#include <glib.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "search.h"
#include "http.h"
#include "support.h"
#include "net.h"
#include "interface.h"
#include "gg.h"

extern GGStatus status;
extern GGConfig config;

extern GtkWidget *wyniki_szukaj;
extern GList *watches;

GList *wyniki_kontakty = NULL;

void gg_search(GGSearch * s)
{
	gchar *query;
	gchar *http_query;
	GGHTTPQuery *q;
	
	gg_remove_watch(GG_SEARCH);
	
    	switch (s->mode) {

	case 0:
		if (s->first_name != NULL)
			s->first_name = gg_urlencode(s->first_name);
		if (s->last_name != NULL)
			s->last_name = gg_urlencode(s->last_name);
		if (s->gender != NULL)
			s->gender = g_strdup("-1");
		if (s->nick != NULL)
			s->nick = gg_urlencode(s->nick);
		if (s->city != NULL)
			s->city = gg_urlencode(s->city);

		query =
		    g_strdup_printf
		    ("Mode=0&FirstName=%s&LastName=%s&Gender=%s&NickName=%s&City=%s&MinBirth=%d&MaxBirth=%d%s",
		     (s->first_name) ? s->first_name : "",
		     (s->last_name) ? s->last_name : "",
		     (s->gender) ? s->gender : "",
		     (s->nick) ? s->nick : "", 
		     (s->city) ? s->city : "",
		     s->minbirth, s->maxbirth,
		     (s->active_only) ? "&ActiveOnly=" : "");
		     cp_to_iso(query);
		break;
	case 1:
		if (s->email != NULL)
			s->email = gg_urlencode(s->email);
		query =
		    g_strdup_printf("Mode=1&Email=%s%s",
				    (s->email) ? s->email : "",
				    (s->
				     active_only) ? "&ActiveOnly=" : "");
		break;
	case 2:
		query =
		    g_strdup_printf("Mode=1&MobilePhone=%s%s",
				    s->mobile,
				    (s->
				     active_only) ? "&ActiveOnly=" : "");
		break;
	case 3:
		query =
		    g_strdup_printf("Mode=3&UserId=%d%s", s->uin,
				    (s->
				     active_only) ? "&ActiveOnly=" : "");
		break;
	default:
		return;
	}

	http_query =
	    g_strdup_printf("POST %s/appsvc/fmpubquery2.asp HTTP/1.0\r\n"
			    "Host: " GG_PUBDIR_HOST "\r\n"
			    "Content-Type: application/x-www-form-urlencoded\r\n"
			    "User-Agent: Mozilla/4.7 (Win 98; I)\r\n"
			    "Content-Length: %d\r\n"
			    "Pragma: no-cache\r\n" "\r\n"
			    "%s\r\n",
			    (config.
			     use_proxy ? "http://" GG_PUBDIR_HOST : ""),
			    strlen(query), query);
	q = g_new0(GGHTTPQuery, 1);
	q->type = GG_SEARCH;
	q->http_query = g_strdup(http_query);

	gg_async_http_query(q, GG_PUBDIR_HOST);

	status.find_stage = GG_FIND_CONNECTING;
	g_free(query);
	g_free(http_query);
}


static void search_error(GGWatch * r)
{
	close(r->fd);
	watches = g_list_remove(watches, r);
	tcp_watch_destroy(r);
	show_error_dialog
	    ("Nie znaleziono nikogo speniajcego podane kryteria");
}
static void abort_search(GGWatch * r)
{
	close(r->fd);
	watches = g_list_remove(watches, r);
	tcp_watch_destroy(r);
#ifdef DEBUG
	g_warning("abort_search(): Abort search!!!");
#endif
}

void gg_http_search_get_res(GGWatch * r)
{
	gint datalen;
	gchar *buf = NULL;
	gchar *record[8];
	GGContact *kontakt;

	if (wyniki_szukaj == NULL || !GTK_IS_WIDGET(wyniki_szukaj)) {
		wyniki_szukaj = create_wyniki_szukaj();
		gtk_widget_show(wyniki_szukaj);
		return;
	}

	if (status.find_stage == GG_FIND_CONNECTING) {
		datalen = http_query_read_head(r->fd);

		if (datalen < 17) {
			search_error(r);
			return;
		}
		status.find_stage = GG_FIND_READING_HEADER;
		g_print("Find status: GG_FIND_READING_HEADER\n");

		return;
	}
	if (status.find_stage == GG_FIND_READING_HEADER) {
		GtkWidget *wyniki_lista;

		buf = g_malloc0(1024);

		wyniki_lista =
		    lookup_widget(wyniki_szukaj, "wyniki_lista");
		if (wyniki_lista == NULL) {
			abort_search(r);
			show_error_dialog
			    ("Nastpi nieoczekiwany bd (nie mog znale widget'a)");
			return;
		}

		gtk_clist_clear(GTK_CLIST(wyniki_lista));
		g_list_free(wyniki_kontakty);

		if (http_read_line(r->fd, buf, 1024)) {
			if (strstr(buf, "query_results:") != NULL)
				status.find_stage = GG_FIND_PARSING;
			else {
				search_error(r);
			}
		} else {
		    search_error(r);
		} 

		g_print("Find status: GG_FIND_PARSING\n");
		g_free(buf);
		return;
	}
	if (status.find_stage == GG_FIND_PARSING) {
		gint i = 0;
		GtkWidget *wyniki_lista;
		gchar tmp[1024];
		gchar *line[4];
		gint row;

		wyniki_lista =
		    lookup_widget(wyniki_szukaj, "wyniki_lista");

		if (wyniki_lista == NULL) {
			abort_search(r);
			show_error_dialog
			    ("Nastpi nieoczekiwany bd (nie mog znale widget'a)");
			return;
		}

		for (i = 0; i < 8; i++) {
			if (http_read_line(r->fd, tmp, 1023)) {
				record[i] = g_strdup(g_strstrip(tmp));
				cp_to_iso(record[i]);
				g_print("record[%d] = %s\n", i, tmp);
			} else {
				g_print("Fsck! i = %d, aborting\n", i);
				abort_search(r);
				return;
			}
		}

		kontakt = g_new0(GGContact, 1);

		kontakt->status = atoi(record[0]);
		kontakt->uin = atoi(record[1]);
		kontakt->first_name =
		    g_strdup((record[2]) ? record[2] : "");
		kontakt->last_name =
		    g_strdup((record[3]) ? record[3] : "");
		kontakt->nick =
		    g_strdup((record[4]) ? record[4] : (record[2] ? record[2] : ""));
		g_print("uin %d\nfirst_name: %s\nlast_name: %s\nnick: %s\n", kontakt->uin,
			kontakt->first_name, kontakt->last_name, kontakt->nick);
		wyniki_kontakty = g_list_append(wyniki_kontakty, kontakt);
		line[0] = g_strdup_printf("%d", kontakt->uin);
		line[1] =
		    g_strjoin(" ",
			      (kontakt->first_name) ? kontakt->
			      first_name : "", kontakt->last_name, NULL);
		line[2] = g_strdup(record[5]);
		line[3] = g_strdup(record[7]);

		row = gtk_clist_append(GTK_CLIST(wyniki_lista), line);
		gtk_clist_set_row_data(GTK_CLIST(wyniki_lista), row,
				       kontakt);
		gg_change_find_kontakt_status(wyniki_lista, kontakt);
		r->timeout = 10;
	}

}
