/* $Id: userstatus.c,v 1.2 2002/06/02 00:17:34 thrull Exp $ */

/*
 * (C) Copyright 2001-2002 Igor Popik <thrull@slackware.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "common.h"
#ifdef USE_GNOME
#include <gnome.h>
#endif

#if USE_APPLET || USE_DOCKAPP || USE_DOCKLET
#include "dock.h"
#endif

#include "gg-types.h"
#include "gg.h"
#include "support.h"
#include "userstatus.h"

extern GGConfig config;
extern GGStatus status;

extern GtkWidget *window;

#ifdef USE_APPLET
extern GtkWidget *applet;
#endif

extern GList *kontakty;
extern GList *sessions;
extern GList *ignore;

#define MAX_CHUNK_SIZE 1024

void set_status_img_status(guint status)
{
	GtkWidget *widget = NULL;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GdkColormap *colormap;
	gchar *filename = NULL;
#if USE_APPLET || USE_DOCKAPP || USE_DOCKLET
	gchar *filename_applet = NULL;
#endif

	switch (status) {
	case GG_STATUS_OFFLINE:
		filename = ONLINE_CON_IMG;
#if USE_APPLET || USE_DOCKAPP || USE_DOCKLET
		filename_applet = OFFLINE_IMG;
#endif
		widget = lookup_widget(window, "online_pixmap");
		g_return_if_fail(widget != NULL);
		break;
	case GG_STATUS_ONLINE:
		filename = ONLINE_IMG;
#if USE_APPLET || USE_DOCKAPP || USE_DOCKLET
		filename_applet = ONLINE_IMG;
#endif
		widget = lookup_widget(window, "online_pixmap");
		g_return_if_fail(widget != NULL);
		break;
	case GG_STATUS_AWAY:
		filename = AWAY_IMG;
#if USE_APPLET || USE_DOCKAPP || USE_DOCKLET
		filename_applet = AWAY_IMG;
#endif
		break;

	case GG_STATUS_INVISIBLE:
#if USE_APPLET || USE_DOCKAPP || USE_DOCKLET
		filename_applet = INVISIBLE_IMG;
#endif
		break;

	default:
		break;
	}


#ifdef USE_APPLET
	applet_change_status_pixmap(filename_applet);
#endif

#ifdef USE_DOCKAPP
	wmgg_change_status_pixmap(filename_applet);
#endif

#ifdef USE_DOCKLET
	docklet_change_status_pixmap(filename_applet);
#endif

	colormap = gtk_widget_get_colormap(window);
	pixmap = gdk_pixmap_colormap_create_from_xpm(NULL, colormap, &mask,
						     NULL,
						     get_pixmap_path
						     (filename));

	if (widget != NULL && pixmap != NULL)
	    gtk_pixmap_set(GTK_PIXMAP(widget), pixmap, mask);
}

/* zmiana statusu i obrazka statusu jednego z naszych kontaktw */
void change_kontakt_status(GGContact * kontakt, guint prev_status)
{
	gint row;
	gchar *nick;
	GtkWidget *lista;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GdkColormap *colormap;
	GGSession *sesja;
	gchar *picname;
	gchar *title=NULL;

	g_return_if_fail(kontakt != NULL);

/*	if (kontakt->status == GG_STATUS_OFFLINE) {
	    remove_kontakt_from_list(kontakt);
	} else {
	    add_kontakt_to_list(kontakt);
	}
*/
	lista = lookup_widget(window, "lista");

	g_return_if_fail(lista != NULL);


	nick =
	    (kontakt->nick) ? kontakt->
	    nick : ((kontakt->first_name) ? kontakt->
		    first_name : g_strdup_printf("%d", kontakt->uin));

	row = gtk_clist_find_row_from_data(GTK_CLIST(lista), kontakt);

	g_return_if_fail(row != -1);

	colormap = gtk_widget_get_colormap(lista);
	g_print("KONTAKT status=%d mobile=%s uin=%d\n", kontakt->status,
		kontakt->mobile, kontakt->uin);

	
	switch (kontakt->status) {
	case GG_STATUS_OFFLINE:
		g_print("KONTAKT status=%d mobile=%s uin=%d\n",
			kontakt->status, kontakt->mobile, kontakt->uin);
		picname =
		    g_strdup((kontakt->
			      mobile ? OFFLINE_MOBILE_IMG : OFFLINE_IMG));
		break;
	case GG_STATUS_ONLINE:
		picname =
		    g_strdup((kontakt->
			      mobile ? ONLINE_MOBILE_IMG : ONLINE_IMG));
		break;
	case GG_STATUS_AWAY:
		picname =
		    g_strdup((kontakt->
			      mobile ? AWAY_MOBILE_IMG : AWAY_IMG));
		break;
	case GG_STATUS_MOBILE_ONLY:
		picname = g_strdup(MOBILE_IMG);
		break;

	default:
		picname =
		    g_strdup((kontakt->
			      mobile ? OFFLINE_MOBILE_IMG : OFFLINE_IMG));
		break;
	}

	pixmap = gdk_pixmap_colormap_create_from_xpm(NULL, colormap, &mask,
						     NULL,
						     get_pixmap_path
						     (picname));
	g_free(picname);

	if (pixmap != NULL) {
		gtk_clist_set_pixtext(GTK_CLIST(lista), row, 0, nick, 3,
				      pixmap, mask);
	}
	gtk_clist_sort(GTK_CLIST(lista));

	if((sesja=gg_find_session(kontakt->uin)))
	{
		if(sesja->chat_window)
		{
			title=g_strdup_printf("Rozmowa z %s (%d) - %s",
					kontakt->nick, kontakt->uin,
					gg_get_status_as_string(kontakt->uin));

			gtk_window_set_title(GTK_WINDOW(sesja->chat_window), title);

			g_free(title);
		}
	}

#ifdef USE_DOCKAPP
	wmgg_contact_change_status(nick, kontakt->status, prev_status);
#endif 
}

void add_kontakt_to_list(GGContact * kontakt)
{
	gint row;
	gchar *nick;
	GtkWidget *lista;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GdkColormap *colormap;

	if (gg_is_ignored(kontakt->uin))
		return;

	lista = lookup_widget(window, "lista");
	g_return_if_fail (lista != NULL);    

	nick =
	    (kontakt->nick) ? kontakt->
	    nick : ((kontakt->first_name) ? kontakt->
		    first_name : g_strdup_printf("%d", kontakt->uin));
	row = gtk_clist_append(GTK_CLIST(lista), &nick);
	gtk_clist_set_row_data(GTK_CLIST(lista), row, kontakt);
	colormap = gtk_widget_get_colormap(lista);
	pixmap = gdk_pixmap_colormap_create_from_xpm(NULL, colormap, &mask,
						     NULL,
						     get_pixmap_path((kontakt->uin ? (kontakt->mobile ? OFFLINE_MOBILE_IMG : OFFLINE_IMG) : MOBILE_IMG)));

	if (pixmap != NULL) {
		gtk_clist_set_pixtext(GTK_CLIST(lista), row, 0, nick, 3,
				      pixmap, mask);
	}
	gtk_clist_sort(GTK_CLIST(lista));
}

void remove_kontakt_from_list(GGContact * kontakt)
{
	gint row;
	GtkWidget *lista;

	lista = lookup_widget(window, "lista");
	g_return_if_fail (lista != NULL);    
	row = gtk_clist_find_row_from_data(GTK_CLIST(lista), kontakt);
	
	gtk_clist_remove(GTK_CLIST(lista), row);
}
